'use strict';

var database = require('@nocobase/database');
var constants = require('../constants');

const aggregators = {
  count: "count",
  sum: "sum",
  avg: "avg",
  min: "min",
  max: "max"
};
var aggregate_default = {
  async run(node, input, processor) {
    const { aggregator, associated, collection, association = {}, params = {} } = node.config;
    const options = processor.getParsedValue(params, node.id);
    const { database: database$1 } = node.constructor;
    const repo = associated ? database$1.getRepository(
      `${association == null ? void 0 : association.associatedCollection}.${association.name}`,
      processor.getParsedValue(association == null ? void 0 : association.associatedKey, node.id)
    ) : database$1.getRepository(collection);
    if (!options.dataType && aggregator === "avg") {
      options.dataType = database.DataTypes.DOUBLE;
    }
    const result = await repo.aggregate({
      ...options,
      method: aggregators[aggregator],
      transaction: processor.transaction
    });
    return {
      result: options.dataType === database.DataTypes.DOUBLE ? Number(result) : result,
      status: constants.JOB_STATUS.RESOLVED
    };
  }
};

module.exports = aggregate_default;
