'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var evaluators = require('@nocobase/evaluators');
var utils = require('@nocobase/utils');
var constants = require('../constants');

const calculators = new utils.Registry();
function equal(a, b) {
  return a == b;
}
function notEqual(a, b) {
  return a != b;
}
function gt(a, b) {
  return a > b;
}
function gte(a, b) {
  return a >= b;
}
function lt(a, b) {
  return a < b;
}
function lte(a, b) {
  return a <= b;
}
calculators.register("equal", equal);
calculators.register("notEqual", notEqual);
calculators.register("gt", gt);
calculators.register("gte", gte);
calculators.register("lt", lt);
calculators.register("lte", lte);
calculators.register("==", equal);
calculators.register("!=", notEqual);
calculators.register(">", gt);
calculators.register(">=", gte);
calculators.register("<", lt);
calculators.register("<=", lte);
function includes(a, b) {
  return a.includes(b);
}
function notIncludes(a, b) {
  return !a.includes(b);
}
function startsWith(a, b) {
  return a.startsWith(b);
}
function notStartsWith(a, b) {
  return !a.startsWith(b);
}
function endsWith(a, b) {
  return a.endsWith(b);
}
function notEndsWith(a, b) {
  return !a.endsWith(b);
}
calculators.register("includes", includes);
calculators.register("notIncludes", notIncludes);
calculators.register("startsWith", startsWith);
calculators.register("notStartsWith", notStartsWith);
calculators.register("endsWith", endsWith);
calculators.register("notEndsWith", notEndsWith);
function calculate(calculation = {}) {
  let fn;
  if (!(calculation.calculator && (fn = calculators.get(calculation.calculator)))) {
    throw new Error(`no calculator function registered for "${calculation.calculator}"`);
  }
  return Boolean(fn(...calculation.operands ?? []));
}
function logicCalculate(calculation) {
  if (!calculation) {
    return true;
  }
  if (typeof calculation["group"] === "object") {
    const method = calculation["group"].type === "and" ? "every" : "some";
    return (calculation["group"].calculations ?? [])[method]((item) => logicCalculate(item));
  }
  return calculate(calculation);
}
var condition_default = {
  async run(node, prevJob, processor) {
    const { engine, calculation, expression, rejectOnFalse } = node.config || {};
    const evaluator = evaluators.evaluators.get(engine);
    let result = true;
    try {
      result = evaluator ? evaluator(expression, processor.getScope(node.id)) : logicCalculate(processor.getParsedValue(calculation, node.id));
    } catch (e) {
      return {
        result: e.toString(),
        status: constants.JOB_STATUS.ERROR
      };
    }
    if (!result && rejectOnFalse) {
      return {
        status: constants.JOB_STATUS.FAILED,
        result
      };
    }
    const job = {
      status: constants.JOB_STATUS.RESOLVED,
      result,
      // TODO(optimize): try unify the building of job
      nodeId: node.id,
      upstreamId: prevJob && prevJob.id || null
    };
    const branchNode = processor.nodes.find((item) => item.upstream === node && Boolean(item.branchIndex) === result);
    if (!branchNode) {
      return job;
    }
    const savedJob = await processor.saveJob(job);
    await processor.run(branchNode, savedJob);
    return null;
  },
  async resume(node, branchJob, processor) {
    const job = processor.findBranchParentJob(branchJob, node);
    if (branchJob.status === constants.JOB_STATUS.RESOLVED) {
      return job;
    }
    return processor.exit(branchJob.status);
  }
};

exports.calculators = calculators;
exports.default = condition_default;
