'use strict';

var constants = require('../constants');

class delay_default {
  constructor(plugin) {
    this.plugin = plugin;
    plugin.app.on("beforeStart", this.load);
    plugin.app.on("beforeStop", this.unload);
  }
  timers = /* @__PURE__ */ new Map();
  load = async () => {
    const { model } = this.plugin.db.getCollection("jobs");
    const jobs = await model.findAll({
      where: {
        status: constants.JOB_STATUS.PENDING
      },
      include: [
        {
          association: "execution",
          attributes: [],
          where: {
            status: constants.EXECUTION_STATUS.STARTED
          },
          required: true
        },
        {
          association: "node",
          attributes: ["config"],
          where: {
            type: "delay"
          },
          required: true
        }
      ]
    });
    jobs.forEach((job) => {
      this.schedule(job);
    });
  };
  unload = () => {
    for (const timer of this.timers.values()) {
      clearTimeout(timer);
    }
    this.timers = /* @__PURE__ */ new Map();
  };
  schedule(job) {
    const now = /* @__PURE__ */ new Date();
    const createdAt = Date.parse(job.createdAt);
    const delay = createdAt + job.node.config.duration - now.getTime();
    if (delay > 0) {
      const trigger = this.trigger.bind(this, job);
      this.timers.set(job.id, setTimeout(trigger, delay));
    } else {
      this.trigger(job);
    }
  }
  async trigger(job) {
    if (!job.execution) {
      job.execution = await job.getExecution();
    }
    if (job.execution.status === constants.EXECUTION_STATUS.STARTED) {
      this.plugin.resume(job);
    }
    if (this.timers.get(job.id)) {
      this.timers.delete(job.id);
    }
  }
  run = async (node, prevJob, processor) => {
    const job = await processor.saveJob({
      status: constants.JOB_STATUS.PENDING,
      result: null,
      nodeId: node.id,
      upstreamId: (prevJob == null ? void 0 : prevJob.id) ?? null
    });
    job.node = node;
    this.schedule(job);
    return processor.exit();
  };
  resume = async (node, prevJob, processor) => {
    const { endStatus } = node.config;
    prevJob.set("status", endStatus);
    return prevJob;
  };
}

module.exports = delay_default;
