'use strict';

var constants = require('../constants');

function getTargetLength(target) {
  let length = 0;
  if (typeof target === "number") {
    if (target < 0) {
      throw new Error("Loop target in number type must be greater than 0");
    }
    length = Math.floor(target);
  } else {
    const targets = (Array.isArray(target) ? target : [target]).filter((t) => t != null);
    length = targets.length;
  }
  return length;
}
var loop_default = {
  async run(node, prevJob, processor) {
    const [branch] = processor.getBranches(node);
    const target = processor.getParsedValue(node.config.target, node.id);
    const length = getTargetLength(target);
    if (!branch || !length) {
      return {
        status: constants.JOB_STATUS.RESOLVED,
        result: 0
      };
    }
    const job = await processor.saveJob({
      status: constants.JOB_STATUS.PENDING,
      // save loop index
      result: 0,
      nodeId: node.id,
      upstreamId: (prevJob == null ? void 0 : prevJob.id) ?? null
    });
    await processor.run(branch, job);
    return null;
  },
  async resume(node, branchJob, processor) {
    const job = processor.findBranchParentJob(branchJob, node);
    const loop = processor.nodesMap.get(job.nodeId);
    const [branch] = processor.getBranches(node);
    const { result, status } = job;
    if (status !== constants.JOB_STATUS.PENDING) {
      return processor.exit();
    }
    const nextIndex = result + 1;
    const target = processor.getParsedValue(loop.config.target, node.id);
    if (branchJob.status > constants.JOB_STATUS.PENDING) {
      job.set({ result: nextIndex });
      const length = getTargetLength(target);
      if (nextIndex < length) {
        await processor.saveJob(job);
        await processor.run(branch, job);
        return null;
      }
    }
    job.set({
      status: branchJob.status
    });
    return job;
  },
  getScope(node, index, processor) {
    const target = processor.getParsedValue(node.config.target, node.id);
    const targets = (Array.isArray(target) ? target : [target]).filter((t) => t != null);
    const length = getTargetLength(target);
    const item = typeof target === "number" ? index : targets[index];
    const result = {
      item,
      index,
      length
    };
    return result;
  }
};

module.exports = loop_default;
