'use strict';

var actions = require('@nocobase/actions');
var constants = require('../../constants');

async function submit(context, next) {
  var _a, _b, _c;
  const repository = actions.utils.getRepositoryFromParams(context);
  const { filterByTk, values } = context.action.params;
  const { currentUser } = context.state;
  if (!currentUser) {
    return context.throw(401);
  }
  const plugin = context.app.pm.get("workflow");
  const instruction = plugin.instructions.get("manual");
  const userJob = await repository.findOne({
    filterByTk,
    // filter: {
    //   userId: currentUser?.id
    // },
    appends: ["job", "node", "execution", "workflow"],
    context
  });
  if (!userJob) {
    return context.throw(404);
  }
  const { forms = {} } = userJob.node.config;
  const [formKey] = Object.keys(values.result ?? {}).filter((key) => key !== "_");
  const actionKey = (_a = values.result) == null ? void 0 : _a._;
  const actionItem = (_c = (_b = forms[formKey]) == null ? void 0 : _b.actions) == null ? void 0 : _c.find((item) => item.key === actionKey);
  if (userJob.status !== constants.JOB_STATUS.PENDING || userJob.job.status !== constants.JOB_STATUS.PENDING || userJob.execution.status !== constants.EXECUTION_STATUS.STARTED || !userJob.workflow.enabled || !actionKey || (actionItem == null ? void 0 : actionItem.status) == null) {
    return context.throw(400);
  }
  userJob.execution.workflow = userJob.workflow;
  const processor = plugin.createProcessor(userJob.execution);
  await processor.prepare();
  const assignees = processor.getParsedValue(userJob.node.config.assignees ?? [], userJob.nodeId);
  if (!assignees.includes(currentUser.id) || userJob.userId !== currentUser.id) {
    return context.throw(403);
  }
  const presetValues = processor.getParsedValue(actionItem.values ?? {}, userJob.nodeId, {
    currentUser: currentUser.toJSON(),
    currentRecord: values.result[formKey],
    currentTime: /* @__PURE__ */ new Date()
  });
  userJob.set({
    status: actionItem.status,
    result: actionItem.status > constants.JOB_STATUS.PENDING ? { [formKey]: Object.assign(values.result[formKey], presetValues), _: actionKey } : Object.assign(userJob.result ?? {}, values.result)
  });
  const handler = instruction.formTypes.get(forms[formKey].type);
  if (handler && userJob.status) {
    await handler.call(instruction, userJob, forms[formKey], processor);
  }
  await userJob.save({ transaction: processor.transaction });
  await processor.exit();
  context.body = userJob;
  context.status = 202;
  await next();
  userJob.job.execution = userJob.execution;
  userJob.job.latestUserJob = userJob;
  processor.logger.info(`manual node (${userJob.nodeId}) action trigger execution (${userJob.execution.id}) to resume`);
  plugin.resume(userJob.job);
}

exports.submit = submit;
