'use strict';

var actions = require('@nocobase/actions');
var utils = require('@nocobase/utils');
var constants = require('../../constants');
var jobsCollection = require('./collecions/jobs');
var usersCollection = require('./collecions/users');
var usersJobsCollection = require('./collecions/users_jobs');
var actions$1 = require('./actions');
var initFormTypes = require('./forms');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var actions__default = /*#__PURE__*/_interopDefault(actions);
var jobsCollection__default = /*#__PURE__*/_interopDefault(jobsCollection);
var usersCollection__default = /*#__PURE__*/_interopDefault(usersCollection);
var usersJobsCollection__default = /*#__PURE__*/_interopDefault(usersJobsCollection);
var initFormTypes__default = /*#__PURE__*/_interopDefault(initFormTypes);

const MULTIPLE_ASSIGNED_MODE = {
  SINGLE: Symbol("single"),
  ALL: Symbol("all"),
  ANY: Symbol("any"),
  ALL_PERCENTAGE: Symbol("all percentage"),
  ANY_PERCENTAGE: Symbol("any percentage")
};
const Modes = {
  [MULTIPLE_ASSIGNED_MODE.SINGLE]: {
    getStatus(distribution, assignees) {
      const done = distribution.find((item) => item.status !== constants.JOB_STATUS.PENDING && item.count > 0);
      return done ? done.status : null;
    }
  },
  [MULTIPLE_ASSIGNED_MODE.ALL]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find((item) => item.status === constants.JOB_STATUS.RESOLVED);
      if (resolved && resolved.count === assignees.length) {
        return constants.JOB_STATUS.RESOLVED;
      }
      const rejected = distribution.find((item) => item.status < constants.JOB_STATUS.PENDING);
      if (rejected && rejected.count) {
        return rejected.status;
      }
      return null;
    }
  },
  [MULTIPLE_ASSIGNED_MODE.ANY]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find((item) => item.status === constants.JOB_STATUS.RESOLVED);
      if (resolved && resolved.count) {
        return constants.JOB_STATUS.RESOLVED;
      }
      const rejectedCount = distribution.reduce(
        (count, item) => item.status < constants.JOB_STATUS.PENDING ? count + item.count : count,
        0
      );
      if (rejectedCount === assignees.length) {
        return constants.JOB_STATUS.REJECTED;
      }
      return null;
    }
  }
};
function getMode(mode) {
  switch (true) {
    case mode === 1:
      return Modes[MULTIPLE_ASSIGNED_MODE.ALL];
    case mode === -1:
      return Modes[MULTIPLE_ASSIGNED_MODE.ANY];
    case mode > 0:
      return Modes[MULTIPLE_ASSIGNED_MODE.ALL_PERCENTAGE];
    case mode < 0:
      return Modes[MULTIPLE_ASSIGNED_MODE.ANY_PERCENTAGE];
    default:
      return Modes[MULTIPLE_ASSIGNED_MODE.SINGLE];
  }
}
class manual_default {
  constructor(plugin) {
    this.plugin = plugin;
    plugin.db.collection(usersJobsCollection__default.default);
    plugin.db.extendCollection(usersCollection__default.default);
    plugin.db.extendCollection(jobsCollection__default.default);
    plugin.app.resource({
      name: "users_jobs",
      actions: {
        list: {
          filter: {
            $or: [
              {
                "workflow.enabled": true
              },
              {
                "workflow.enabled": false,
                status: {
                  $ne: constants.JOB_STATUS.PENDING
                }
              }
            ]
          },
          handler: actions__default.default.list
        },
        submit: actions$1.submit
      }
    });
    initFormTypes__default.default(this);
  }
  formTypes = new utils.Registry();
  async run(node, prevJob, processor) {
    const { mode, ...config } = node.config;
    const assignees = [...new Set(processor.getParsedValue(config.assignees, node.id) || [])];
    const job = await processor.saveJob({
      status: constants.JOB_STATUS.PENDING,
      result: mode ? [] : null,
      nodeId: node.id,
      upstreamId: (prevJob == null ? void 0 : prevJob.id) ?? null
    });
    const UserJobModel = processor.options.plugin.db.getModel("users_jobs");
    await UserJobModel.bulkCreate(
      assignees.map((userId) => ({
        userId,
        jobId: job.id,
        nodeId: node.id,
        executionId: job.executionId,
        workflowId: node.workflowId,
        status: constants.JOB_STATUS.PENDING
      })),
      {
        transaction: processor.transaction
      }
    );
    return job;
  }
  async resume(node, job, processor) {
    var _a;
    const { assignees = [], mode } = node.config;
    const UserJobModel = processor.options.plugin.db.getModel("users_jobs");
    const distribution = await UserJobModel.count({
      where: {
        jobId: job.id
      },
      group: ["status"],
      transaction: processor.transaction
    });
    const submitted = distribution.reduce(
      (count, item) => item.status !== constants.JOB_STATUS.PENDING ? count + item.count : count,
      0
    );
    const status = job.status || (getMode(mode).getStatus(distribution, assignees) ?? constants.JOB_STATUS.PENDING);
    const result = mode ? (submitted || 0) / assignees.length : ((_a = job.latestUserJob) == null ? void 0 : _a.result) ?? job.result;
    processor.logger.debug(`manual resume job and next status: ${status}`);
    job.set({
      status,
      result
    });
    return job;
  }
}

module.exports = manual_default;
