'use strict';

var actions = require('@nocobase/actions');
var constants = require('../constants');
var utils = require('../utils');

var query_default = {
  async run(node, input, processor) {
    const { collection, multiple, params = {}, failOnEmpty = false } = node.config;
    const repo = node.constructor.database.getRepository(collection);
    const {
      page = actions.DEFAULT_PAGE,
      pageSize = actions.DEFAULT_PER_PAGE,
      sort = [],
      ...options
    } = processor.getParsedValue(params, node.id);
    const appends = options.appends ? Array.from(
      options.appends.reduce((set, field) => {
        set.add(field.split(".")[0]);
        set.add(field);
        return set;
      }, /* @__PURE__ */ new Set())
    ) : options.appends;
    const result = await (multiple ? repo.find : repo.findOne).call(repo, {
      ...options,
      ...actions.utils.pageArgsToLimitArgs(page, pageSize),
      sort: sort.filter((item) => item.field).map((item) => {
        var _a;
        return `${((_a = item.direction) == null ? void 0 : _a.toLowerCase()) === "desc" ? "-" : ""}${item.field}`;
      }),
      appends,
      transaction: processor.transaction
    });
    if (failOnEmpty && (multiple ? !result.length : !result)) {
      return {
        result,
        status: constants.JOB_STATUS.FAILED
      };
    }
    return {
      result: utils.toJSON(result),
      status: constants.JOB_STATUS.RESOLVED
    };
  }
};

module.exports = query_default;
