'use strict';

var axios = require('axios');
var constants = require('../constants');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var axios__default = /*#__PURE__*/_interopDefault(axios);

async function request(config) {
  const { url, method = "POST", data, timeout = 5e3 } = config;
  const headers = (config.headers ?? []).reduce((result, header) => {
    if (header.name.toLowerCase() === "content-type") {
      return result;
    }
    return Object.assign(result, { [header.name]: header.value });
  }, {});
  const params = (config.params ?? []).reduce(
    (result, param) => Object.assign(result, { [param.name]: param.value }),
    {}
  );
  headers["Content-Type"] = "application/json";
  return axios__default.default.request({
    url,
    method,
    headers,
    params,
    data,
    timeout
  });
}
class request_default {
  constructor(plugin) {
    this.plugin = plugin;
  }
  async run(node, prevJob, processor) {
    const job = await processor.saveJob({
      status: constants.JOB_STATUS.PENDING,
      nodeId: node.id,
      upstreamId: (prevJob == null ? void 0 : prevJob.id) ?? null
    });
    const config = processor.getParsedValue(node.config, node.id);
    request(config).then((response) => {
      job.set({
        status: constants.JOB_STATUS.RESOLVED,
        result: response.data
      });
    }).catch((error) => {
      job.set({
        status: constants.JOB_STATUS.FAILED,
        result: error.isAxiosError ? error.toJSON() : error.message
      });
    }).finally(() => {
      processor.logger.info(`request (#${node.id}) response received, status: ${job.get("status")}`);
      this.plugin.resume(job);
    });
    processor.logger.info(`request (#${node.id}) sent to "${config.url}", waiting for response...`);
    return processor.exit();
  }
  async resume(node, job, processor) {
    const { ignoreFail } = node.config;
    if (ignoreFail) {
      job.set("status", constants.JOB_STATUS.RESOLVED);
    }
    return job;
  }
}

module.exports = request_default;
