'use strict';

var lodash = require('lodash');
var _ = require('.');
var database = require('@nocobase/database');

class FormTrigger extends _.Trigger {
  constructor(plugin) {
    super(plugin);
    plugin.app.resourcer.use(this.middleware);
    plugin.app.actions({
      ["workflows:trigger"]: this.triggerAction
    });
  }
  triggerAction = async (context, next) => {
    const { triggerWorkflows } = context.action.params;
    if (!triggerWorkflows) {
      return context.throw(400);
    }
    context.status = 202;
    await next();
    this.trigger(context);
  };
  middleware = async (context, next) => {
    await next();
    const { resourceName, actionName } = context.action;
    if (resourceName === "workflows" && actionName === "trigger" || !["create", "update"].includes(actionName)) {
      return;
    }
    this.trigger(context);
  };
  async trigger(context) {
    const { triggerWorkflows, values } = context.action.params;
    if (!triggerWorkflows) {
      return;
    }
    const { currentUser } = context.state;
    const triggers = triggerWorkflows.split(",").map((trigger) => trigger.split("!"));
    const workflowRepo = this.plugin.db.getRepository("workflows");
    const workflows = await workflowRepo.find({
      filter: {
        key: triggers.map((trigger) => trigger[0]),
        current: true,
        type: "form",
        enabled: true
      }
    });
    workflows.forEach((workflow) => {
      var _a;
      const trigger = triggers.find((trigger2) => trigger2[0] == workflow.key);
      if ((_a = context.body) == null ? void 0 : _a.data) {
        const { data } = context.body;
        (Array.isArray(data) ? data : [data]).forEach(async (row) => {
          let payload = row;
          if (trigger[1]) {
            const paths = trigger[1].split(".");
            for await (const field of paths) {
              if (payload.get(field)) {
                payload = payload.get(field);
              } else {
                const association = database.modelAssociationByKey(payload, field);
                payload = await payload[association.accessors.get]();
              }
            }
          }
          const { collection, appends = [] } = workflow.config;
          const model = payload.constructor;
          if (collection !== model.collection.name) {
            return;
          }
          if (appends.length) {
            payload = await model.collection.repository.findOne({
              filterByTk: payload.get(model.primaryKeyAttribute),
              appends
            });
          }
          this.plugin.trigger(workflow, { data: payload, user: currentUser });
        });
      } else {
        const data = trigger[1] ? lodash.get(values, trigger[1]) : values;
        this.plugin.trigger(workflow, {
          data,
          user: currentUser
        });
      }
    });
  }
  on(workflow) {
  }
  off(workflow) {
  }
}

module.exports = FormTrigger;
